#ifndef MEMTRC_INCLUDED
#define MEMTRC_INCLUDED

/*
	-------------------------------------------
	 C Memory management surveillance routines
	-------------------------------------------

	Author: Martin Vorlaender (mv@pdv-systeme.de)
	        based on code written by
	        Diethard Wettrau  (dw@pdv-systeme.de)

	See MEMTRC.C for instructions on usage.
*/

#ifdef NDEBUG

#define memtrc_notify_alloc( addr, size )
#define memtrc_flush_list( logname )

#else /* ndef NDEBUG */

#include <stdlib.h>	/* size_t */

/**********************  Redirection macros  **************************/

#ifdef malloc
#	undef malloc
#endif
#define malloc( size ) \
	memtrc_malloc( (size_t)(size), __FILE__, __LINE__ )

#ifdef free
#	undef free
#endif
#define free( addr ) \
	memtrc_free( (void**)&(addr), __FILE__, __LINE__ )

#ifdef calloc
#	undef calloc
#endif
#define calloc( items, size ) \
	memtrc_calloc( (size_t)(items), (size_t)(size), __FILE__, __LINE__ )

#ifdef cfree
#	undef cfree
#endif
#define cfree( addr ) \
	memtrc_cfree( (void**)&(addr), __FILE__, __LINE__ )

#ifdef realloc
#	undef realloc
#endif
#define realloc( addr, size ) \
	memtrc_realloc( (void**)&(addr), (size_t)(size), __FILE__, __LINE__ )

#ifdef strdup
#	undef strdup
#endif
#define strdup( str ) \
	memtrc_strdup( (str), __FILE__, __LINE__ )

#ifdef XtMalloc
#	undef XtMalloc
#endif
#define XtMalloc( size ) \
	memtrc_malloc( (size_t)(size), __FILE__, __LINE__ )

#ifdef XtFree
#	undef XtFree
#endif
#define XtFree( addr ) \
	memtrc_free( (void**)&(addr), __FILE__, __LINE__ )

#ifdef XtCalloc
#	undef XtCalloc
#endif
#define XtCalloc( items, size ) \
	memtrc_calloc( (size_t)(items), (size_t)(size), __FILE__, __LINE__ )

#ifdef XtRealloc
#	undef XtRealloc
#endif
#define XtRealloc( addr, size ) \
	memtrc_realloc( (void**)&(addr), (size_t)(size), __FILE__, __LINE__ )

#ifdef XtNewString
#	undef XtNewString
#endif
#define XtNewString( str ) \
	memtrc_strdup( (str), __FILE__, __LINE__ )

#ifdef memtrc_notify_alloc
#	undef memtrc_notify_alloc
#endif
#define memtrc_notify_alloc( addr, size ) \
	memtrc_notify_alloc2( (void*)(addr), (size_t)(size), __FILE__, __LINE__ )

/**********************  Interface routines  ***************************/

extern void memtrc_flush_list
(
	char *log_file_name  /* if NULL: output to stderr */
);

/* memtrc_notify_alloc() is #define'd above. The declaration would be:
extern void memtrc_notify_alloc
(
	void   *addr,
	size_t size
);
*/

/**********************  Internal routines  ***************************/

extern void *memtrc_malloc
(
	size_t size,
	char   *file,
	long   line
);

extern void *memtrc_calloc
(
	size_t items,
	size_t size,
	char   *file,
	long   line
);

extern char *memtrc_newstring
(
	char *str,
	char *file,
	long line
);

extern void memtrc_free
(
	void **addrPtr,
	char *file,
	long line
);
	
extern void memtrc_cfree
(
	void **addrPtr,
	char *file,
	long line
);

extern void memtrc_notify_alloc2
(
	void   *addr,
	size_t size,
	char   *file,
	long   line
);

#endif /* NDEBUG */

#endif /* MEMTRC_INCLUDED */
